// Binary Coded Decimal
// 
// Simplified code as example for a BCD converter.
//
// by feyd//godX.de

#include <cstdint>
#include <stdio.h>
#include <stdlib.h>

class CBCD
{
public:
	CBCD(int nDigits) : m_nDigits(nDigits) {};
	virtual ~CBCD() {};

	unsigned int FromBCD(const uint8_t* pBCD)
	{
		unsigned int nNonBCD = 0;
		for(int n=0; n<m_nDigits; n++)
		{
			nNonBCD *= 10;
			nNonBCD += (pBCD[n>>1]>>(((n^1)&1)*4))&15;
		}
		return nNonBCD;
	}

	void ToBCD(unsigned int nNonBCD, uint8_t* pBCD)
	{
		unsigned int nBase = 1;
		for(int n=m_nDigits-1; n>=0; n--)
		{
			if(n&1)
				pBCD[n>>1] = 0;	
			pBCD[n>>1] |= ((nNonBCD/nBase)%10)<<(((n^1)&1)*4);
			nBase *= 10;
		}
	}
private:
	int m_nDigits;
};

// Application entry point (from libc)
int main(int argc, const char* argv[])
{
	printf("Binary coded decimal test...\r\n\r\n");

	if(argc>1)
	{
		uint8_t buffer[4];
		for(int n=1; n<argc; n++)
		{
			unsigned int nOriginal = (unsigned int)atoi(argv[n]);
			CBCD(8).ToBCD(nOriginal, buffer);
			unsigned int nConvertedBack = CBCD(8).FromBCD(buffer);
			printf("  Original: %10u ( 0x%08x) ->     packed BCD:            (%02x %02x %02x %02x)\r\n", nOriginal, nOriginal, buffer[0], buffer[1], buffer[2], buffer[3]);
			printf("packed BCD:            (%02x %02x %02x %02x) -> Converted back: %10u ( 0x%08x)\r\n", buffer[0], buffer[1], buffer[2], buffer[3], nConvertedBack, nConvertedBack);
			printf("\r\n");
		}
	} else {
		printf("not enough arguments\r\n");
	}
	return 0;
}

